@@ -7,19 +7,20 @@ logger_format = logging.Formatter('%(levelname)s:%(threadName)s %(name)s - %(asc
 stream_handler = logging.StreamHandler()
 stream_handler.setFormatter(logger_format)
 main_logger.addHandler(stream_handler)
-logger = main_logger.getChild(__name__)
+
+logger = logging.getLogger(__name__)
 
 # Setting up data directory
 DATA_DIR_KEY = 'TIMEWISE_DATA'
 if DATA_DIR_KEY in os.environ:
-    data_dir = os.environ[DATA_DIR_KEY]
+    data_dir = os.path.expanduser(os.environ[DATA_DIR_KEY])
 else:
     logger.warning(f'{DATA_DIR_KEY} not set! Using home directory.')
     data_dir = os.path.expanduser('~/')
 
 BIGDATA_DIR_KEY = 'TIMEWISE_BIGDATA'
 if BIGDATA_DIR_KEY in os.environ:
-    bigdata_dir = os.environ[BIGDATA_DIR_KEY]
+    bigdata_dir = os.path.expanduser(os.environ[BIGDATA_DIR_KEY])
     logger.info(f"Using bigdata directory {bigdata_dir}")
 else:
     bigdata_dir = None
@@ -31,10 +32,10 @@ cache_dir = os.path.join(data_dir, 'cache')
 
 for d in [data_dir, output_dir, plots_dir, cache_dir]:
     if not os.path.isdir(d):
-        os.mkdir(d)
+        os.mkdir(os.path.abspath(d))
 
 
 def backoff_hndlr(details):
     logger.info("Backing off {wait:0.1f} seconds after {tries} tries "
                 "calling function {target} with args {args} and kwargs "
-                "{kwargs}".format(**details))
\ No newline at end of file
+                "{kwargs}".format(**details))
