@@ -69,19 +69,16 @@ class WISEDataDESYCluster(WiseDataByVisit):
         self._n_cluster_jobs_per_chunk = None
         self._storage_dir = None
 
-        # use a lock to prevent multiple threads to load and write to disc simultaneously
-        self.disc_lock = threading.Lock()
-
         # status attributes
         self.start_time = None
         self._total_tasks = None
         self._done_tasks = None
 
-        self._tap_queue = queue.Queue()
-        self._cluster_queue = queue.Queue()
-        self._io_queue = queue.PriorityQueue()
-        self._io_queue_done = queue.Queue()
-        self._combining_queue = queue.Queue()
+        self._tap_queue = None
+        self._cluster_queue = None
+        self._io_queue = None
+        self._io_queue_done = None
+        self._combining_queue = None
 
     # ---------------------------------------------------------------------------------- #
     # START using gzip to compress the data when saving     #
@@ -227,9 +224,6 @@ class WISEDataDESYCluster(WiseDataByVisit):
 
         service = 'tap'
 
-        # set up queue
-        self.queue = queue.Queue()
-
         # set up dictionary to store jobs in
         self.tap_jobs = {t: dict() for t in tables}
 
@@ -248,6 +242,15 @@ class WISEDataDESYCluster(WiseDataByVisit):
         self._overwrite = overwrite
         self._storage_dir = storage_directory
 
+        # --------------------------- set up queues --------------------------- #
+
+        self.queue = queue.Queue()
+        self._tap_queue = queue.Queue()
+        self._cluster_queue = queue.Queue()
+        self._io_queue = queue.PriorityQueue()
+        self._io_queue_done = queue.Queue()
+        self._combining_queue = queue.Queue()
+
         # --------------------------- starting threads --------------------------- #
 
         tap_threads = [threading.Thread(target=self._tap_thread, daemon=True, name=f"TAPThread{_}")
@@ -288,6 +291,14 @@ class WISEDataDESYCluster(WiseDataByVisit):
         self._combining_queue.join()
         logger.debug('combining done')
 
+        # unset queues
+        self.queue = None
+        self._tap_queue = None
+        self._cluster_queue = None
+        self._io_queue = None
+        self._io_queue_done = None
+        self._combining_queue = None
+
     @backoff.on_exception(
         backoff.expo,
         vo.dal.exceptions.DALServiceError,
@@ -473,7 +484,11 @@ class WISEDataDESYCluster(WiseDataByVisit):
     def _combining_thread(self):
         logger.debug(f'started combining thread')
         while True:
-            chunk = self._combining_queue.get(block=True)
+
+            try:
+                chunk = self._combining_queue.get(block=True)
+            except AttributeError:  # when self._combining_queue is None, meaning it was reset in main thread
+                break
             logger.debug(f"combining chunk {chunk}")
 
             try:
