@@ -21,6 +21,7 @@ from astropy import constants
 from astropy.cosmology import Planck18
 from astropy.io import ascii
 from astropy.table import Table
+from astropy.coordinates import SkyCoord
 
 from timewise.general import cache_dir, plots_dir, output_dir, logger_format, backoff_hndlr
 from timewise.utils import StableTAPService
@@ -205,7 +206,7 @@ class WISEDataBase(abc.ABC):
         self._split_chunk_key = '__chunk'
         self._cached_raw_photometry_prefix = 'raw_photometry'
         self.tap_jobs = None
-        self.queue = queue.Queue()
+        self.queue = None
         self.clear_unbinned_photometry_when_binning = False
         self._cached_final_products = {
             'lightcurves': dict(),
@@ -548,7 +549,7 @@ class WISEDataBase(abc.ABC):
 
     def get_photometric_data(self, tables=None, perc=1, wait=0, service=None, nthreads=100,
                              chunks=None, overwrite=True, remove_chunks=False, query_type='positional',
-                             skip_download=False):
+                             skip_download=False, mask_by_position=False):
         """
         Load photometric data from the IRSA server for the matched sample. The result will be saved under
 
@@ -574,6 +575,8 @@ class WISEDataBase(abc.ABC):
         :type query_type: str
         :param skip_download: if `True` skip downloading and only do binning
         :type skip_download: bool
+        :param mask_by_position: if `True` mask single exposures that are too far away from the bulk
+        :type mask_by_position: bool
         """
 
         mag = True
@@ -618,7 +621,7 @@ class WISEDataBase(abc.ABC):
         else:
             logger.info("skipping download, assume data is already downloaded.")
 
-        self._select_individual_lightcurves_and_bin(service=service, chunks=chunks)
+        self._select_individual_lightcurves_and_bin(service=service, chunks=chunks, mask_by_position=mask_by_position)
         for c in chunks:
             self.calculate_metadata(service=service, chunk_number=c, overwrite=True)
 
@@ -1089,12 +1092,14 @@ class WISEDataBase(abc.ABC):
     #     select individual lightcurves and bin
     # ----------------------------------------------------------------------
 
-    def _select_individual_lightcurves_and_bin(self, ncpu=35, service='tap', chunks=None):
+    def _select_individual_lightcurves_and_bin(self, ncpu=35, service='tap', chunks=None, mask_by_position=False):
         logger.info('selecting individual lightcurves and bin ...')
         ncpu = min(self.n_chunks, ncpu)
         logger.debug(f"using {ncpu} CPUs")
         chunk_list = list(range(self.n_chunks)) if not chunks else chunks
         service_list = [service] * len(chunk_list)
+        jobID_list = [None] * len(chunk_list)
+        pos_mask_list = [mask_by_position] * len(chunk_list)
         logger.debug(f"multiprocessing arguments: chunks: {chunk_list}, service: {service_list}")
 
         while True:
@@ -1113,7 +1118,7 @@ class WISEDataBase(abc.ABC):
                 tqdm.tqdm(
                     p.starmap(
                         self._subprocess_select_and_bin,
-                        zip(service_list, chunk_list)
+                        zip(service_list, chunk_list, jobID_list, pos_mask_list)
                     ),
                     total=self.n_chunks,
                     desc='select and bin'
@@ -1122,7 +1127,7 @@ class WISEDataBase(abc.ABC):
             p.close()
             p.join()
         else:
-            r = list(map(self._subprocess_select_and_bin, service_list, chunk_list))
+            r = list(map(self._subprocess_select_and_bin, service_list, chunk_list, jobID_list, pos_mask_list))
 
     def get_unbinned_lightcurves(self, chunk_number, clear=False):
         """
@@ -1152,7 +1157,7 @@ class WISEDataBase(abc.ABC):
 
         return lightcurves
 
-    def _subprocess_select_and_bin(self, service, chunk_number=None, jobID=None):
+    def _subprocess_select_and_bin(self, service, chunk_number=None, jobID=None, mask_by_position=False):
         # run through the ids and bin the lightcurves
         if service == 'tap':
             lightcurves = self.get_unbinned_lightcurves(chunk_number, clear=self.clear_unbinned_photometry_when_binning)
@@ -1177,10 +1182,19 @@ class WISEDataBase(abc.ABC):
             logger.info(f"Starting data product for {len(indices)} indices.")
             data_product = self._start_data_product(parent_sample_indices=indices)
 
+        if mask_by_position:
+            position_mask = self.get_position_mask(service, chunk_number)
+        else:
+            position_mask = None
+
         for parent_sample_entry_id in tqdm.tqdm(indices, desc="binning"):
             m = lightcurves[self._tap_orig_id_key] == parent_sample_entry_id
             lightcurve = lightcurves[m]
 
+            if position_mask is not None:
+                pos_m = pd.Series(position_mask[str(parent_sample_entry_id)]).values
+                lightcurve = lightcurve[pos_m]
+
             if len(lightcurve) < 1:
                 logger.warning(f"No data for {parent_sample_entry_id}")
                 continue
@@ -1447,6 +1461,94 @@ class WISEDataBase(abc.ABC):
     # END GET PHOTOMETRY DATA       #
     ###########################################################################################################
 
+    ###########################################################################################################
+    # START MAKE POSITIONAL MASK        #
+    #####################################
+
+    @staticmethod
+    def calculate_position_mask(lightcurve):
+        """
+        Create a mask that based on the position of the single exposures.
+        Find the median position and find the 90%-quantile of datapoints from that.
+        Then, calculate the standard deviation of the separation from the median position and keep
+        all datapoints within five times that.
+
+        :param lightcurve: unstacked lightcurve
+        :type lightcurve: pd.DataFrame
+        :return: positional mask
+        :rtype: np.ndarray
+        """
+        coords = SkyCoord(lightcurve.ra, lightcurve.dec, unit="deg")
+
+        # we can use the standard median for Dec
+        med_offset_dec = np.median(coords.dec)
+        # We have to do a weighted median for RA
+        w = np.sin(np.deg2rad(coords.dec)) ** 2
+        sort_inds = np.argsort(coords.ra)
+        cum_w = np.cumsum(w[sort_inds])
+        cutoff = np.sum(w) / 2
+        med_offset_ra = coords.ra[sort_inds][cum_w >= cutoff][0]
+        med_pos = SkyCoord(med_offset_ra, med_offset_dec)
+
+        # find the 90% closest datapoints
+        sep = coords.separation(med_pos)
+        sep90 = np.quantile(sep, 0.9)
+        sep_mask = sep < sep90
+
+        # keep datapoints within 3 sigma
+        sig = max([np.std(sep[sep_mask]), 0.2*u.arcsec])
+        keep_mask = sep <= 5 * sig
+        return pd.Series(keep_mask).to_dict()
+
+    def get_position_mask(self, service, chunk_number):
+        """
+        Get the position mask for a chunk
+
+        :param service: The service that was used to download the data, either of `gator` or `tap`
+        :type service: str
+        :param chunk_number: chunk number
+        :type chunk_number: int
+        :returns: position masks
+        :rtype: dict
+        """
+
+        logger.info(f"getting position masks for {service}, chunk {chunk_number}")
+        fn = os.path.join(self.cache_dir, "position_masks", f"{service}_chunk{chunk_number}.json")
+
+        if not os.path.isfile(fn):
+            logger.debug(f"No file {fn}. Calculating position masks.")
+
+            if service == "tap":
+                unbinned_lcs = self.get_unbinned_lightcurves(chunk_number)
+            elif service == "gator":
+                unbinned_lcs = self._get_unbinned_lightcurves_gator(chunk_number)
+            else:
+                raise ValueError(f"Service must be one of 'gator' or 'tap', not {service}!")
+
+            position_masks = dict()
+
+            for i in unbinned_lcs[self._tap_orig_id_key].unique():
+                lightcurve = unbinned_lcs[unbinned_lcs[self._tap_orig_id_key] == i]
+                position_masks[str(i)] = self.calculate_position_mask(lightcurve)
+
+            d = os.path.dirname(fn)
+            if not os.path.isdir(d):
+                os.makedirs(d)
+
+            with open(fn, "w") as f:
+                json.dump(position_masks, f)
+
+        else:
+            logger.debug(f"loading {fn}")
+            with open(fn, "r") as f:
+                position_masks = json.load(f)
+
+        return position_masks
+
+    #####################################
+    # END MAKE POSITIONAL MASK          #
+    ###########################################################################################################
+
     ###########################################################################################################
     # START MAKE PLOTTING FUNCTIONS     #
     #####################################
@@ -1526,11 +1628,24 @@ class WISEDataBase(abc.ABC):
                                 markeredgecolor='k', ecolor='k', capsize=2)
                     ax.scatter(lightcurve.mean_mjd[ul_mask], lightcurve[f"{b}{self.mean_key}_{lum_key}"][ul_mask],
                                marker='v', c=self.band_plot_colors[b], alpha=0.7, s=2)
+
                 if not isinstance(unbinned_lc, type(None)):
-                    ax.errorbar(unbinned_lc.mjd, unbinned_lc[f"{b}_{lum_key}"],
-                                yerr=unbinned_lc[f"{b}_{lum_key}{self.error_key_ext}"],
-                                label=f"{b} unbinned", ls='', marker='o', c=self.band_plot_colors[b], markersize=4,
-                                alpha=0.3)
+                    m = ~unbinned_lc[f"{b}_{lum_key}"].isna()
+                    ul_mask = unbinned_lc[f"{b}_{lum_key}{self.error_key_ext}"].isna()
+
+                    tot_m = m & ~ul_mask
+                    if np.any(tot_m):
+                        ax.errorbar(unbinned_lc.mjd[tot_m], unbinned_lc[f"{b}_{lum_key}"][tot_m],
+                                    yerr=unbinned_lc[f"{b}_{lum_key}{self.error_key_ext}"][tot_m],
+                                    label=f"{b} unbinned", ls='', marker='o', c=self.band_plot_colors[b], markersize=4,
+                                    alpha=0.3)
+
+                    single_ul_m = m & ul_mask
+                    if np.any(single_ul_m):
+                        label = f"{b} unbinned upper limits" if not np.any(tot_m) else ""
+                        ax.scatter(unbinned_lc.mjd[single_ul_m], unbinned_lc[f"{b}_{lum_key}"][single_ul_m],
+                                   marker="d", c=self.band_plot_colors[b], alpha=0.3, s=1, label=label)
+
             except KeyError as e:
                 raise KeyError(f"Could not find brightness key {e}!")
 
